@@ -10,6 +10,7 @@
  ****************************************************************************/
 
 #include <ArborX_DistributedTree.hpp>
+#include <ArborX_PairIndexRank.hpp>
 #include <ArborX_Version.hpp>
 
 #include <Kokkos_Core.hpp>
@@ -20,6 +21,7 @@
 #include <chrono>
 #include <cmath> // sqrt, cbrt
 #include <iomanip>
+#include <iostream> // cout
 #include <numeric>
 #include <random>
 #include <utility>
@@ -168,56 +170,10 @@ public:
   }
 };
 
-template <typename DeviceType>
-struct NearestNeighborsSearches
-{
-  Kokkos::View<ArborX::Point *, DeviceType> points;
-  int k;
-};
-template <typename DeviceType>
-struct RadiusSearches
-{
-  Kokkos::View<ArborX::Point *, DeviceType> points;
-  double radius;
-};
-
-template <typename DeviceType>
-struct ArborX::AccessTraits<RadiusSearches<DeviceType>, ArborX::PredicatesTag>
-{
-  using memory_space = typename DeviceType::memory_space;
-  static KOKKOS_FUNCTION std::size_t
-  size(RadiusSearches<DeviceType> const &pred)
-  {
-    return pred.points.extent(0);
-  }
-  static KOKKOS_FUNCTION auto get(RadiusSearches<DeviceType> const &pred,
-                                  std::size_t i)
-  {
-    return ArborX::intersects(ArborX::Sphere{pred.points(i), pred.radius});
-  }
-};
-
-template <typename DeviceType>
-struct ArborX::AccessTraits<NearestNeighborsSearches<DeviceType>,
-                            ArborX::PredicatesTag>
-{
-  using memory_space = typename DeviceType::memory_space;
-  static KOKKOS_FUNCTION std::size_t
-  size(NearestNeighborsSearches<DeviceType> const &pred)
-  {
-    return pred.points.extent(0);
-  }
-  static KOKKOS_FUNCTION auto
-  get(NearestNeighborsSearches<DeviceType> const &pred, std::size_t i)
-  {
-    return ArborX::nearest(pred.points(i), pred.k);
-  }
-};
-
 namespace bpo = boost::program_options;
 
 template <class NO>
-int main_(std::vector<std::string> const &args, const MPI_Comm comm)
+int main_(std::vector<std::string> const &args, MPI_Comm const comm)
 {
   TimeMonitor time_monitor;
 
@@ -293,10 +249,10 @@ int main_(std::vector<std::string> const &args, const MPI_Comm comm)
   }
 
   Kokkos::View<ArborX::Point *, DeviceType> random_values(
-      Kokkos::view_alloc(Kokkos::WithoutInitializing, "Testing::values"),
+      Kokkos::view_alloc(Kokkos::WithoutInitializing, "Benchmark::values"),
       n_values);
   Kokkos::View<ArborX::Point *, DeviceType> random_queries(
-      Kokkos::view_alloc(Kokkos::WithoutInitializing, "Testing::queries"),
+      Kokkos::view_alloc(Kokkos::WithoutInitializing, "Benchmark::queries"),
       n_queries);
   {
     double a = 0.;
@@ -358,7 +314,7 @@ int main_(std::vector<std::string> const &args, const MPI_Comm comm)
     // The boxes in which the points are placed have side length two, centered
     // around offset_[xyz] and scaled by a.
     Kokkos::View<ArborX::Point *, DeviceType> random_points(
-        Kokkos::view_alloc(Kokkos::WithoutInitializing, "Testing::points"),
+        Kokkos::view_alloc(Kokkos::WithoutInitializing, "Benchmark::points"),
         std::max(n_values, n_queries));
     auto random_points_host = Kokkos::create_mirror_view(random_points);
     for (int i = 0; i < random_points.extent_int(0); ++i)
@@ -398,25 +354,11 @@ int main_(std::vector<std::string> const &args, const MPI_Comm comm)
     }
   }
 
-  Kokkos::View<ArborX::Box *, DeviceType> bounding_boxes(
-      Kokkos::view_alloc(Kokkos::WithoutInitializing,
-                         "Testing::bounding_boxes"),
-      n_values);
-  Kokkos::parallel_for(
-      "bvh_driver:construct_bounding_boxes",
-      Kokkos::RangePolicy<ExecutionSpace>(0, n_values), KOKKOS_LAMBDA(int i) {
-        double const x = random_values(i)[0];
-        double const y = random_values(i)[1];
-        double const z = random_values(i)[2];
-        bounding_boxes(i) = {{{x - 1., y - 1., z - 1.}},
-                             {{x + 1., y + 1., z + 1.}}};
-      });
-
   auto construction = time_monitor.getNewTimer("construction");
   MPI_Barrier(comm);
   construction->start();
-  ArborX::DistributedTree<MemorySpace> distributed_tree(comm, ExecutionSpace{},
-                                                        bounding_boxes);
+  ArborX::DistributedTree<MemorySpace, ArborX::Point> distributed_tree(
+      comm, ExecutionSpace{}, random_values);
   construction->stop();
 
   std::ostream &os = std::cout;
@@ -425,17 +367,16 @@ int main_(std::vector<std::string> const &args, const MPI_Comm comm)
 
   if (perform_knn_search)
   {
-    Kokkos::View<int *, DeviceType> offsets("Testing::offsets", 0);
-    Kokkos::View<ArborX::PairIndexRank *, DeviceType> values("Testing::values",
-                                                             0);
+    Kokkos::View<int *, DeviceType> offsets("Benchmark::offsets", 0);
+    Kokkos::View<ArborX::Point *, DeviceType> values("Benchmark::values", 0);
 
     auto knn = time_monitor.getNewTimer("knn");
     MPI_Barrier(comm);
     knn->start();
     distributed_tree.query(
         ExecutionSpace{},
-        NearestNeighborsSearches<DeviceType>{random_queries, n_neighbors},
-        values, offsets);
+        ArborX::Experimental::make_nearest(random_queries, n_neighbors), values,
+        offsets);
     knn->stop();
 
     if (comm_rank == 0)
@@ -446,41 +387,34 @@ int main_(std::vector<std::string> const &args, const MPI_Comm comm)
   {
     // Radius is computed so that the number of results per query for a
     // uniformly distributed primitives in a [-a,a]^d box is approximately
-    // n_neighbors. The primivites are boxes and not points. Thus, the radius
-    // we would have chosen for the case of point primitives has to be adjusted
-    // to account for box-box interaction. The radius is decreased by an
-    // average of the lengths of a half-edge and a half-diagonal to account for
-    // that (approximately). An exact calculation would require computing
-    // an integral.
-    double r = 0.;
+    // n_neighbors.
+    float r = 0.;
     switch (partition_dim)
     {
     case 1:
-      // Derivation (first term): n_values*(2*r)/(2a) = n_neighbors
-      r = static_cast<double>(n_neighbors) - 1.;
+      // Derivation: n_values*(2*r)/(2a) = n_neighbors
+      r = static_cast<float>(n_neighbors);
       break;
     case 2:
-      // Derivation (first term): n_values*(M_PI*r^2)/(2a)^2 = n_neighbors
-      r = std::sqrt(static_cast<double>(n_neighbors) * 4. / M_PI) -
-          (1. + std::sqrt(2.)) / 2;
+      // Derivation: n_values*(pi*r^2)/(2a)^2 = n_neighbors
+      r = std::sqrt(n_neighbors * 4.f / Kokkos::numbers::pi_v<float>);
       break;
     case 3:
-      // Derivation (first term): n_values*(4/3*M_PI*r^3)/(2a)^3 = n_neighbors
-      r = std::cbrt(static_cast<double>(n_neighbors) * 6. / M_PI) -
-          (1. + std::cbrt(3.)) / 2;
+      // Derivation: n_values*(4/3*pi*r^3)/(2a)^3 = n_neighbors
+      r = std::cbrt(n_neighbors * 6.f / Kokkos::numbers::pi_v<float>);
       break;
     }
 
     Kokkos::View<int *, DeviceType> offsets("Testing::offsets", 0);
-    Kokkos::View<ArborX::PairIndexRank *, DeviceType> values("Testing::values",
-                                                             0);
+    Kokkos::View<ArborX::Point *, DeviceType> values("Testing::values", 0);
 
     auto radius = time_monitor.getNewTimer("radius");
     MPI_Barrier(comm);
     radius->start();
-    distributed_tree.query(ExecutionSpace{},
-                           RadiusSearches<DeviceType>{random_queries, r},
-                           values, offsets);
+    distributed_tree.query(
+        ExecutionSpace{},
+        ArborX::Experimental::make_intersects(random_queries, r), values,
+        offsets);
     radius->stop();
 
     if (comm_rank == 0)
@@ -502,7 +436,8 @@ int main(int argc, char *argv[])
   {
     std::cout << "ArborX version: " << ArborX::version() << std::endl;
     std::cout << "ArborX hash   : " << ArborX::gitCommitHash() << std::endl;
-    std::cout << "Kokkos version: " << KokkosExt::version() << std::endl;
+    std::cout << "Kokkos version: " << ArborX::Details::KokkosExt::version()
+              << std::endl;
   }
 
   // Strip "--help" and "--kokkos-help" from the flags passed to Kokkos if we
@@ -602,8 +537,7 @@ int main(int argc, char *argv[])
     if (node == "hip")
     {
 #ifdef KOKKOS_ENABLE_HIP
-      using Node = Kokkos::Device<Kokkos::Experimental::HIP,
-                                  Kokkos::Experimental::HIPSpace>;
+      using Node = Kokkos::Device<Kokkos::HIP, Kokkos::HIPSpace>;
       main_<Node>(pass_further, comm);
 #else
       throw std::runtime_error("HIP node type is disabled");
