@@ -15,6 +15,7 @@
 #include <ArborX_Config.hpp>
 
 #include <ArborX_Box.hpp>
+#include <ArborX_BruteForce.hpp>
 #ifdef ARBORX_ENABLE_MPI
 #include <ArborX_DistributedTree.hpp>
 #endif
@@ -22,7 +23,13 @@
 #include <ArborX_Exception.hpp>
 #include <ArborX_LinearBVH.hpp>
 #include <ArborX_Point.hpp>
+#include <ArborX_PredicateHelpers.hpp>
 #include <ArborX_Predicates.hpp>
 #include <ArborX_Sphere.hpp>
+// FIXME: we include ArborX_DetailsUtils.hpp only for backward compatibility for
+// users using deprecated functions in ArborX namespace (min, max,
+// adjacentDifference, ...). This header should be removed when we remove those
+// functions.
+#include <ArborX_DetailsUtils.hpp>
 
 #endif
