@@ -17,10 +17,14 @@
 #include <ArborX_DetailsFDBSCAN.hpp>
 #include <ArborX_DetailsFDBSCANDenseBox.hpp>
 #include <ArborX_DetailsHalfTraversal.hpp>
+#include <ArborX_DetailsKokkosExtAccessibilityTraits.hpp>
+#include <ArborX_DetailsKokkosExtStdAlgorithms.hpp>
 #include <ArborX_DetailsSortUtils.hpp>
+#include <ArborX_DetailsUtils.hpp> // sortObjects
 #include <ArborX_HyperBox.hpp>
 #include <ArborX_HyperSphere.hpp>
 #include <ArborX_LinearBVH.hpp>
+#include <ArborX_PredicateHelpers.hpp>
 #include <ArborX_Sphere.hpp>
 
 namespace ArborX
@@ -49,26 +53,20 @@ struct DBSCANCorePoints
   }
 };
 
-template <typename Primitives>
-struct PrimitivesWithRadius
-{
-  Primitives _primitives;
-  float _r;
-};
-
 struct WithinRadiusGetter
 {
   float _r;
 
-  template <typename Box>
-  KOKKOS_FUNCTION auto operator()(Box const &box) const
+  template <typename Point, typename Index>
+  KOKKOS_FUNCTION auto
+  operator()(PairValueIndex<Point, Index> const &pair) const
   {
-    static_assert(GeometryTraits::is_box<Box>::value);
+    static_assert(GeometryTraits::is_point_v<Point>);
 
-    constexpr int dim = GeometryTraits::dimension_v<Box>;
+    constexpr int dim = GeometryTraits::dimension_v<Point>;
     auto const &hyper_point =
         reinterpret_cast<ExperimentalHyperGeometry::Point<dim> const &>(
-            box.minCorner());
+            pair.value);
     using ArborX::intersects;
     return intersects(ExperimentalHyperGeometry::Sphere<dim>{hyper_point, _r});
   }
@@ -84,14 +82,12 @@ struct PrimitivesWithRadiusReorderedAndFiltered
 
 // Mixed primitives consist of a set of boxes corresponding to dense cells,
 // followed by boxes corresponding to points in non-dense cells.
-template <typename PointPrimitives, typename DenseCellOffsets,
-          typename CellIndices, typename Permutation>
+template <typename Points, typename DenseCellOffsets, typename CellIndices,
+          typename Permutation>
 struct MixedBoxPrimitives
 {
-  PointPrimitives _point_primitives;
-  CartesianGrid<GeometryTraits::dimension_v<typename AccessTraitsHelper<
-      AccessTraits<PointPrimitives, PrimitivesTag>>::type>>
-      _grid;
+  Points _points;
+  CartesianGrid<GeometryTraits::dimension_v<typename Points::value_type>> _grid;
   DenseCellOffsets _dense_cell_offsets;
   int _num_points_in_dense_cells; // to avoid lastElement() in AccessTraits
   CellIndices _sorted_cell_indices;
@@ -100,41 +96,12 @@ struct MixedBoxPrimitives
 
 } // namespace Details
 
-template <typename Primitives>
-struct AccessTraits<Details::PrimitivesWithRadius<Primitives>, PredicatesTag>
-{
-  using PrimitivesAccess = AccessTraits<Primitives, PrimitivesTag>;
-
-  using memory_space = typename PrimitivesAccess::memory_space;
-  using Predicates = Details::PrimitivesWithRadius<Primitives>;
-
-  static KOKKOS_FUNCTION size_t size(Predicates const &w)
-  {
-    return PrimitivesAccess::size(w._primitives);
-  }
-  static KOKKOS_FUNCTION auto get(Predicates const &w, size_t i)
-  {
-    auto const &point = PrimitivesAccess::get(w._primitives, i);
-    constexpr int dim =
-        GeometryTraits::dimension_v<std::decay_t<decltype(point)>>;
-    // FIXME reinterpret_cast is dangerous here if access traits return user
-    // point structure (e.g., struct MyPoint { float y; float x; })
-    auto const &hyper_point =
-        reinterpret_cast<ExperimentalHyperGeometry::Point<dim> const &>(point);
-    return attach(
-        intersects(ExperimentalHyperGeometry::Sphere<dim>{hyper_point, w._r}),
-        (int)i);
-  }
-};
-
 template <typename Primitives, typename PermuteFilter>
 struct AccessTraits<Details::PrimitivesWithRadiusReorderedAndFiltered<
                         Primitives, PermuteFilter>,
                     PredicatesTag>
 {
-  using PrimitivesAccess = AccessTraits<Primitives, PrimitivesTag>;
-
-  using memory_space = typename PrimitivesAccess::memory_space;
+  using memory_space = typename Primitives::memory_space;
   using Predicates =
       Details::PrimitivesWithRadiusReorderedAndFiltered<Primitives,
                                                         PermuteFilter>;
@@ -146,7 +113,7 @@ struct AccessTraits<Details::PrimitivesWithRadiusReorderedAndFiltered<
   static KOKKOS_FUNCTION auto get(Predicates const &w, size_t i)
   {
     int index = w._filter(i);
-    auto const &point = PrimitivesAccess::get(w._primitives, index);
+    auto const &point = w._primitives(index);
     constexpr int dim =
         GeometryTraits::dimension_v<std::decay_t<decltype(point)>>;
     // FIXME reinterpret_cast is dangerous here if access traits return user
@@ -159,13 +126,13 @@ struct AccessTraits<Details::PrimitivesWithRadiusReorderedAndFiltered<
   }
 };
 
-template <typename PointPrimitives, typename MixedOffsets, typename CellIndices,
+template <typename Points, typename MixedOffsets, typename CellIndices,
           typename Permutation>
-struct AccessTraits<Details::MixedBoxPrimitives<PointPrimitives, MixedOffsets,
-                                                CellIndices, Permutation>,
-                    ArborX::PrimitivesTag>
+struct AccessTraits<
+    Details::MixedBoxPrimitives<Points, MixedOffsets, CellIndices, Permutation>,
+    ArborX::PrimitivesTag>
 {
-  using Primitives = Details::MixedBoxPrimitives<PointPrimitives, MixedOffsets,
+  using Primitives = Details::MixedBoxPrimitives<Points, MixedOffsets,
                                                  CellIndices, Permutation>;
   static KOKKOS_FUNCTION std::size_t size(Primitives const &w)
   {
@@ -194,11 +161,9 @@ struct AccessTraits<Details::MixedBoxPrimitives<PointPrimitives, MixedOffsets,
     // For a primitive corresponding to a point in a non-dense cell, use that
     // point. But first, figure out its index, which requires some
     // computations.
-    using Access = AccessTraits<PointPrimitives, PrimitivesTag>;
-
     i = (i - num_dense_primitives) + w._num_points_in_dense_cells;
 
-    auto const &point = Access::get(w._point_primitives, w._permute(i));
+    auto const &point = w._points(w._permute(i));
     constexpr int dim =
         GeometryTraits::dimension_v<std::decay_t<decltype(point)>>;
     // FIXME reinterpret_cast is dangerous here if access traits return user
@@ -248,8 +213,10 @@ dbscan(ExecutionSpace const &exec_space, Primitives const &primitives,
 {
   Kokkos::Profiling::pushRegion("ArborX::DBSCAN");
 
-  using Access = AccessTraits<Primitives, PrimitivesTag>;
-  using MemorySpace = typename Access::memory_space;
+  namespace KokkosExt = ArborX::Details::KokkosExt;
+
+  using Points = Details::AccessValues<Primitives, PrimitivesTag>;
+  using MemorySpace = typename Points::memory_space;
 
   static_assert(
       KokkosExt::is_accessible_from<MemorySpace, ExecutionSpace>::value,
@@ -266,32 +233,37 @@ dbscan(ExecutionSpace const &exec_space, Primitives const &primitives,
   using UnionFind = Details::UnionFind<MemorySpace>;
 #endif
 
-  constexpr int dim = GeometryTraits::dimension_v<
-      typename Details::AccessTraitsHelper<Access>::type>;
+  using Point = typename Points::value_type;
+  static_assert(GeometryTraits::is_point_v<Point>);
+  constexpr int dim = GeometryTraits::dimension_v<Point>;
   using Box = ExperimentalHyperGeometry::Box<dim>;
 
   bool const is_special_case = (core_min_size == 2);
 
   bool const verbose = parameters._verbose;
 
-  int const n = Access::size(primitives);
+  Points points{primitives}; // NOLINT
+  int const n = points.size();
 
   Kokkos::View<int *, MemorySpace> num_neigh("ArborX::DBSCAN::num_neighbors",
                                              0);
 
-  Kokkos::View<int *, MemorySpace> labels(
-      Kokkos::view_alloc(exec_space, Kokkos::WithoutInitializing,
-                         "ArborX::DBSCAN::labels"),
-      n);
-  ArborX::iota(exec_space, labels);
+  Kokkos::View<int *, MemorySpace> labels("ArborX::DBSCAN::labels", 0);
 
   if (parameters._implementation == DBSCAN::Implementation::FDBSCAN)
   {
     // Build the tree
     Kokkos::Profiling::pushRegion("ArborX::DBSCAN::tree_construction");
-    BasicBoundingVolumeHierarchy<MemorySpace, Box> bvh(exec_space, primitives);
+    BoundingVolumeHierarchy<MemorySpace, PairValueIndex<Point>> bvh(
+        exec_space, Experimental::attach_indices(points));
     Kokkos::Profiling::popRegion();
 
+    // Initialize labels after the hierarchy construction to lower memory high
+    // water mark
+    Kokkos::resize(Kokkos::view_alloc(exec_space, Kokkos::WithoutInitializing),
+                   labels, n);
+    KokkosExt::iota(exec_space, labels);
+
     Kokkos::Profiling::pushRegion("ArborX::DBSCAN::clusters");
     if (is_special_case)
     {
@@ -314,8 +286,8 @@ dbscan(ExecutionSpace const &exec_space, Primitives const &primitives,
     }
     else
     {
-      auto const predicates =
-          Details::PrimitivesWithRadius<Primitives>{primitives, eps};
+      auto const predicates = ArborX::Experimental::attach_indices(
+          ArborX::Experimental::make_intersects(points, eps));
 
       // Determine core points
       Kokkos::Profiling::pushRegion("ArborX::DBSCAN::clusters::num_neigh");
@@ -350,15 +322,17 @@ dbscan(ExecutionSpace const &exec_space, Primitives const &primitives,
     Kokkos::Profiling::pushRegion("ArborX::DBSCAN::dense_cells");
     Box bounds;
     Details::TreeConstruction::calculateBoundingBoxOfTheScene(
-        exec_space, primitives, bounds);
+        exec_space,
+        Details::Indexables<Points, Details::DefaultIndexableGetter>{
+            points, Details::DefaultIndexableGetter{}},
+        bounds);
 
     // The cell length is chosen to be eps/sqrt(dimension), so that any two
     // points within the same cell are within eps distance of each other.
     float const h = eps / std::sqrt(dim);
     Details::CartesianGrid<dim> const grid(bounds, h);
 
-    auto cell_indices =
-        Details::computeCellIndices(exec_space, primitives, grid);
+    auto cell_indices = Details::computeCellIndices(exec_space, points, grid);
 
     auto permute = Details::sortObjects(exec_space, cell_indices);
     auto &sorted_cell_indices = cell_indices; // alias
@@ -402,6 +376,10 @@ dbscan(ExecutionSpace const &exec_space, Primitives const &primitives,
              num_dense_cells + num_points_in_sparse_cells);
     }
 
+    Kokkos::resize(Kokkos::view_alloc(exec_space, Kokkos::WithoutInitializing),
+                   labels, n);
+    KokkosExt::iota(exec_space, labels);
+
     Details::unionFindWithinEachDenseCell(exec_space, dense_sorted_cell_indices,
                                           permute, UnionFind{labels});
 
@@ -409,12 +387,17 @@ dbscan(ExecutionSpace const &exec_space, Primitives const &primitives,
 
     // Build the tree
     Kokkos::Profiling::pushRegion("ArborX::DBSCAN::tree_construction");
-    BasicBoundingVolumeHierarchy<MemorySpace, Box> bvh(
-        exec_space,
-        Details::MixedBoxPrimitives<Primitives, decltype(dense_cell_offsets),
-                                    decltype(cell_indices), decltype(permute)>{
-            primitives, grid, dense_cell_offsets, num_points_in_dense_cells,
-            sorted_cell_indices, permute});
+    Details::MixedBoxPrimitives<Points, decltype(dense_cell_offsets),
+                                decltype(cell_indices), decltype(permute)>
+        mixed_primitives{points,
+                         grid,
+                         dense_cell_offsets,
+                         num_points_in_dense_cells,
+                         sorted_cell_indices,
+                         permute};
+
+    BoundingVolumeHierarchy<MemorySpace, PairValueIndex<Box>> bvh(
+        exec_space, Experimental::attach_indices(mixed_primitives));
 
     Kokkos::Profiling::popRegion();
 
@@ -425,15 +408,14 @@ dbscan(ExecutionSpace const &exec_space, Primitives const &primitives,
       // Perform the queries and build clusters through callback
       using CorePoints = Details::CCSCorePoints;
       Kokkos::Profiling::pushRegion("ArborX::DBSCAN::clusters::query");
-      auto const predicates =
-          Details::PrimitivesWithRadius<Primitives>{primitives, eps};
-      bvh.query(
-          exec_space, predicates,
-          Details::FDBSCANDenseBoxCallback<UnionFind, CorePoints, Primitives,
-                                           decltype(dense_cell_offsets),
-                                           decltype(permute)>{
-              labels, CorePoints{}, primitives, dense_cell_offsets, exec_space,
-              permute, eps});
+      auto const predicates = Experimental::attach_indices(
+          Experimental::make_intersects(points, eps));
+      bvh.query(exec_space, predicates,
+                Details::FDBSCANDenseBoxCallback<UnionFind, CorePoints, Points,
+                                                 decltype(dense_cell_offsets),
+                                                 decltype(permute)>{
+                    labels, CorePoints{}, points, dense_cell_offsets,
+                    exec_space, permute, eps});
       Kokkos::Profiling::popRegion();
     }
     else
@@ -454,13 +436,12 @@ dbscan(ExecutionSpace const &exec_space, Primitives const &primitives,
 
       auto const sparse_predicates =
           Details::PrimitivesWithRadiusReorderedAndFiltered<
-              Primitives, decltype(sparse_permute)>{primitives, eps,
-                                                    sparse_permute};
+              Points, decltype(sparse_permute)>{points, eps, sparse_permute};
       bvh.query(exec_space, sparse_predicates,
-                Details::CountUpToN_DenseBox<MemorySpace, Primitives,
+                Details::CountUpToN_DenseBox<MemorySpace, Points,
                                              decltype(dense_cell_offsets),
                                              decltype(permute)>(
-                    num_neigh, primitives, dense_cell_offsets, permute,
+                    num_neigh, points, dense_cell_offsets, permute,
                     core_min_size, eps, core_min_size));
       Kokkos::Profiling::popRegion();
 
@@ -468,15 +449,14 @@ dbscan(ExecutionSpace const &exec_space, Primitives const &primitives,
 
       // Perform the queries and build clusters through callback
       Kokkos::Profiling::pushRegion("ArborX::DBSCAN::clusters::query");
-      auto const predicates =
-          Details::PrimitivesWithRadius<Primitives>{primitives, eps};
-      bvh.query(
-          exec_space, predicates,
-          Details::FDBSCANDenseBoxCallback<UnionFind, CorePoints, Primitives,
-                                           decltype(dense_cell_offsets),
-                                           decltype(permute)>{
-              labels, CorePoints{num_neigh, core_min_size}, primitives,
-              dense_cell_offsets, exec_space, permute, eps});
+      auto const predicates = Experimental::attach_indices(
+          Experimental::make_intersects(points, eps));
+      bvh.query(exec_space, predicates,
+                Details::FDBSCANDenseBoxCallback<UnionFind, CorePoints, Points,
+                                                 decltype(dense_cell_offsets),
+                                                 decltype(permute)>{
+                    labels, CorePoints{num_neigh, core_min_size}, points,
+                    dense_cell_offsets, exec_space, permute, eps});
       Kokkos::Profiling::popRegion();
     }
   }
