@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 2017-2022 by the ArborX authors                            *
+ * Copyright (c) 2017-2023 by the ArborX authors                            *
  * All rights reserved.                                                     *
  *                                                                          *
  * This file is part of the ArborX library. ArborX is                       *
@@ -12,10 +12,10 @@
 #ifndef ARBORX_DETAIL_CRS_GRAPH_WRAPPER_IMPL_HPP
 #define ARBORX_DETAIL_CRS_GRAPH_WRAPPER_IMPL_HPP
 
-#include <ArborX_AccessTraits.hpp>
 #include <ArborX_Box.hpp>
 #include <ArborX_Callbacks.hpp>
 #include <ArborX_DetailsBatchedQueries.hpp>
+#include <ArborX_DetailsKokkosExtStdAlgorithms.hpp>
 #include <ArborX_DetailsKokkosExtViewHelpers.hpp>
 #include <ArborX_DetailsPermutedData.hpp>
 #include <ArborX_HyperBox.hpp>
@@ -50,8 +50,8 @@ struct FirstPassNoBufferOptimizationTag
 struct SecondPassTag
 {};
 
-template <typename PassTag, typename Predicates, typename Callback,
-          typename OutputView, typename CountView, typename PermutedOffset>
+template <typename PassTag, typename Callback, typename OutputView,
+          typename CountView, typename PermutedOffset>
 struct InsertGenerator
 {
   Callback _callback;
@@ -60,68 +60,51 @@ struct InsertGenerator
   PermutedOffset _permuted_offset;
 
   using ValueType = typename OutputView::value_type;
-  using Access = AccessTraits<Predicates, PredicatesTag>;
-  using PredicateType = typename AccessTraitsHelper<Access>::type;
 
-  template <typename U = PassTag,
-            std::enable_if_t<std::is_same<U, FirstPassTag>{}> * = nullptr>
-  KOKKOS_FUNCTION auto operator()(PredicateType const &predicate,
-                                  int primitive_index) const
+  template <typename Predicate, typename Value>
+  KOKKOS_FUNCTION auto operator()(Predicate const &predicate,
+                                  Value const &value) const
   {
     auto const predicate_index = getData(predicate);
     auto const &raw_predicate = getPredicate(predicate);
-    // With permutation, we access offset in random manner, and
-    // _offset(permutated_predicate_index+1) may be in a completely different
-    // place. Instead, use pointers to get the correct value for the buffer
-    // size. For this reason, also take a reference for offset.
-    auto const &offset = _permuted_offset(predicate_index);
-    auto const buffer_size = *(&offset + 1) - offset;
     auto &count = _counts(predicate_index);
 
-    return _callback(raw_predicate, primitive_index,
-                     [&](ValueType const &value) {
-                       int count_old = Kokkos::atomic_fetch_add(&count, 1);
-                       if (count_old < buffer_size)
-                         _out(offset + count_old) = value;
-                     });
-  }
+    if constexpr (std::is_same_v<PassTag, FirstPassTag>)
+    {
+      // With permutation, we access offset in random manner, and
+      // _offset(permutated_predicate_index+1) may be in a completely different
+      // place. Instead, use pointers to get the correct value for the buffer
+      // size. For this reason, also take a reference for offset.
+      auto const &offset = _permuted_offset(predicate_index);
+      auto const buffer_size = *(&offset + 1) - offset;
 
-  template <
-      typename U = PassTag,
-      std::enable_if_t<std::is_same<U, FirstPassNoBufferOptimizationTag>{}> * =
-          nullptr>
-  KOKKOS_FUNCTION auto operator()(PredicateType const &predicate,
-                                  int primitive_index) const
-  {
-    auto const predicate_index = getData(predicate);
-    auto const &raw_predicate = getPredicate(predicate);
+      return _callback(raw_predicate, value, [&](ValueType const &v) {
+        int count_old = Kokkos::atomic_fetch_add(&count, 1);
+        if (count_old < buffer_size)
+          _out(offset + count_old) = v;
+      });
+    }
+    else if constexpr (std::is_same_v<PassTag,
+                                      FirstPassNoBufferOptimizationTag>)
+    {
+      return _callback(raw_predicate, value, [&](ValueType const &) {
+        Kokkos::atomic_increment(&count);
+      });
+    }
+    else
+    {
+      static_assert(std::is_same_v<PassTag, SecondPassTag>);
+      // we store offsets in counts, and offset(permute(i)) = counts(i)
+      auto &offset = count;
 
-    auto &count = _counts(predicate_index);
-
-    return _callback(raw_predicate, primitive_index, [&](ValueType const &) {
-      Kokkos::atomic_increment(&count);
-    });
-  }
-
-  template <typename U = PassTag,
-            std::enable_if_t<std::is_same<U, SecondPassTag>{}> * = nullptr>
-  KOKKOS_FUNCTION auto operator()(PredicateType const &predicate,
-                                  int primitive_index) const
-  {
-    auto const predicate_index = getData(predicate);
-    auto const &raw_predicate = getPredicate(predicate);
-
-    // we store offsets in counts, and offset(permute(i)) = counts(i)
-    auto &offset = _counts(predicate_index);
-
-    // TODO: there is a tradeoff here between skipping computation offset +
-    // count, and atomic increment of count. I think atomically incrementing
-    // offset is problematic for OpenMP as you potentially constantly steal
-    // cache lines.
-    return _callback(raw_predicate, primitive_index,
-                     [&](ValueType const &value) {
-                       _out(Kokkos::atomic_fetch_add(&offset, 1)) = value;
-                     });
+      // TODO: there is a tradeoff here between skipping computation offset +
+      // count, and atomic increment of count. I think atomically incrementing
+      // offset is problematic for OpenMP as you potentially constantly steal
+      // cache lines.
+      return _callback(raw_predicate, value, [&](ValueType const &v) {
+        _out(Kokkos::atomic_fetch_add(&offset, 1)) = v;
+      });
+    }
   }
 };
 
@@ -141,8 +124,7 @@ void queryImpl(ExecutionSpace const &space, Tree const &tree,
 
   static_assert(Kokkos::is_execution_space<ExecutionSpace>{});
 
-  using Access = AccessTraits<Predicates, PredicatesTag>;
-  auto const n_queries = Access::size(predicates);
+  auto const n_queries = predicates.size();
 
   Kokkos::Profiling::pushRegion("ArborX::CrsGraphWrapper::two_pass");
 
@@ -165,15 +147,14 @@ void queryImpl(ExecutionSpace const &space, Tree const &tree,
   {
     tree.query(
         space, permuted_predicates,
-        InsertGenerator<FirstPassTag, PermutedPredicates, Callback, OutputView,
-                        CountView, PermutedOffset>{callback, out, counts,
-                                                   permuted_offset},
+        InsertGenerator<FirstPassTag, Callback, OutputView, CountView,
+                        PermutedOffset>{callback, out, counts, permuted_offset},
         ArborX::Experimental::TraversalPolicy().setPredicateSorting(false));
 
     // Detecting overflow is a local operation that needs to be done for every
     // index. We allow individual buffer sizes to differ, so it's not as easy
     // as computing max counts.
-    int overflow_int = 0;
+    int overflow_int;
     Kokkos::parallel_reduce(
         "ArborX::CrsGraphWrapper::compute_overflow",
         Kokkos::RangePolicy<ExecutionSpace>(space, 0, n_queries),
@@ -187,7 +168,7 @@ void queryImpl(ExecutionSpace const &space, Tree const &tree,
 
     if (!overflow)
     {
-      int n_results = 0;
+      int n_results;
       Kokkos::parallel_reduce(
           "ArborX::CrsGraphWrapper::compute_underflow",
           Kokkos::RangePolicy<ExecutionSpace>(space, 0, n_queries),
@@ -200,9 +181,9 @@ void queryImpl(ExecutionSpace const &space, Tree const &tree,
   {
     tree.query(
         space, permuted_predicates,
-        InsertGenerator<FirstPassNoBufferOptimizationTag, PermutedPredicates,
-                        Callback, OutputView, CountView, PermutedOffset>{
-            callback, out, counts, permuted_offset},
+        InsertGenerator<FirstPassNoBufferOptimizationTag, Callback, OutputView,
+                        CountView, PermutedOffset>{callback, out, counts,
+                                                   permuted_offset},
         ArborX::Experimental::TraversalPolicy().setPredicateSorting(false));
     // This may not be true, but it does not matter. As long as we have
     // (n_results == 0) check before second pass, this value is not used.
@@ -225,7 +206,7 @@ void queryImpl(ExecutionSpace const &space, Tree const &tree,
       "ArborX::CrsGraphWrapper::copy_counts_to_offsets",
       Kokkos::RangePolicy<ExecutionSpace>(space, 0, n_queries),
       KOKKOS_LAMBDA(int const i) { permuted_offset(i) = counts(i); });
-  exclusivePrefixSum(space, offset);
+  KokkosExt::exclusive_scan(space, offset, offset, 0);
 
   int const n_results = KokkosExt::lastElement(space, offset);
 
@@ -262,9 +243,8 @@ void queryImpl(ExecutionSpace const &space, Tree const &tree,
 
     tree.query(
         space, permuted_predicates,
-        InsertGenerator<SecondPassTag, PermutedPredicates, Callback, OutputView,
-                        CountView, PermutedOffset>{callback, out, counts,
-                                                   permuted_offset},
+        InsertGenerator<SecondPassTag, Callback, OutputView, CountView,
+                        PermutedOffset>{callback, out, counts, permuted_offset},
         ArborX::Experimental::TraversalPolicy().setPredicateSorting(false));
 
     Kokkos::Profiling::popRegion();
@@ -307,15 +287,13 @@ struct Iota
 
 template <typename Tag, typename ExecutionSpace, typename Predicates,
           typename OffsetView, typename OutView>
-std::enable_if_t<std::is_same<Tag, SpatialPredicateTag>{} ||
-                 std::is_same<Tag, Experimental::OrderedSpatialPredicateTag>{}>
+std::enable_if_t<std::is_same_v<Tag, SpatialPredicateTag> ||
+                 std::is_same_v<Tag, Experimental::OrderedSpatialPredicateTag>>
 allocateAndInitializeStorage(Tag, ExecutionSpace const &space,
                              Predicates const &predicates, OffsetView &offset,
                              OutView &out, int buffer_size)
 {
-  using Access = AccessTraits<Predicates, PredicatesTag>;
-
-  auto const n_queries = Access::size(predicates);
+  auto const n_queries = predicates.size();
   KokkosExt::reallocWithoutInitializing(space, offset, n_queries + 1);
 
   buffer_size = std::abs(buffer_size);
@@ -324,7 +302,7 @@ allocateAndInitializeStorage(Tag, ExecutionSpace const &space,
 
   if (buffer_size != 0)
   {
-    exclusivePrefixSum(space, offset);
+    KokkosExt::exclusive_scan(space, offset, offset, 0);
 
     // Use calculation for the size to avoid calling lastElement(space, offset)
     // as it will launch an extra kernel to copy to host.
@@ -334,22 +312,20 @@ allocateAndInitializeStorage(Tag, ExecutionSpace const &space,
 
 template <typename Tag, typename ExecutionSpace, typename Predicates,
           typename OffsetView, typename OutView>
-std::enable_if_t<std::is_same<Tag, NearestPredicateTag>{}>
+std::enable_if_t<std::is_same_v<Tag, NearestPredicateTag>>
 allocateAndInitializeStorage(Tag, ExecutionSpace const &space,
                              Predicates const &predicates, OffsetView &offset,
                              OutView &out, int /*buffer_size*/)
 {
-  using Access = AccessTraits<Predicates, PredicatesTag>;
-
-  auto const n_queries = Access::size(predicates);
+  auto const n_queries = predicates.size();
   KokkosExt::reallocWithoutInitializing(space, offset, n_queries + 1);
 
   Kokkos::parallel_for(
       "ArborX::CrsGraphWrapper::query::nearest::"
       "scan_queries_for_numbers_of_nearest_neighbors",
       Kokkos::RangePolicy<ExecutionSpace>(space, 0, n_queries),
-      KOKKOS_LAMBDA(int i) { offset(i) = getK(Access::get(predicates, i)); });
-  exclusivePrefixSum(space, offset);
+      KOKKOS_LAMBDA(int i) { offset(i) = getK(predicates(i)); });
+  KokkosExt::exclusive_scan(space, offset, offset, 0);
 
   KokkosExt::reallocWithoutInitializing(space, out,
                                         KokkosExt::lastElement(space, offset));
@@ -360,8 +336,8 @@ allocateAndInitializeStorage(Tag, ExecutionSpace const &space,
 template <typename Tag, typename Tree, typename ExecutionSpace,
           typename Predicates, typename OutputView, typename OffsetView,
           typename Callback>
-std::enable_if_t<!is_tagged_post_callback<Callback>{} &&
-                 Kokkos::is_view<OutputView>{} && Kokkos::is_view<OffsetView>{}>
+std::enable_if_t<!is_tagged_post_callback<Callback>::value &&
+                 Kokkos::is_view_v<OutputView> && Kokkos::is_view_v<OffsetView>>
 queryDispatch(Tag, Tree const &tree, ExecutionSpace const &space,
               Predicates const &predicates, Callback const &callback,
               OutputView &out, OffsetView &offset,
@@ -371,7 +347,7 @@ queryDispatch(Tag, Tree const &tree, ExecutionSpace const &space,
   using MemorySpace = typename Tree::memory_space;
   using DeviceType = Kokkos::Device<ExecutionSpace, MemorySpace>;
 
-  check_valid_callback(callback, predicates, out);
+  check_valid_callback<typename Tree::value_type>(callback, predicates, out);
 
   std::string profiling_prefix = "ArborX::CrsGraphWrapper::query::";
   if constexpr (std::is_same_v<Tag, SpatialPredicateTag>)
@@ -410,7 +386,8 @@ queryDispatch(Tag, Tree const &tree, ExecutionSpace const &space,
     Kokkos::Profiling::pushRegion(profiling_prefix + "::compute_permutation");
     using bounding_volume_type = std::decay_t<decltype(tree.bounds())>;
     ExperimentalHyperGeometry::Box<
-        GeometryTraits::dimension_v<bounding_volume_type>>
+        GeometryTraits::dimension_v<bounding_volume_type>,
+        typename GeometryTraits::coordinate_type_t<bounding_volume_type>>
         scene_bounding_box{};
     using namespace Details;
     expand(scene_bounding_box, tree.bounds());
@@ -434,7 +411,7 @@ queryDispatch(Tag, Tree const &tree, ExecutionSpace const &space,
 
 template <typename Tag, typename Tree, typename ExecutionSpace,
           typename Predicates, typename Indices, typename Offset>
-inline std::enable_if_t<Kokkos::is_view<Indices>{} && Kokkos::is_view<Offset>{}>
+inline std::enable_if_t<Kokkos::is_view_v<Indices> && Kokkos::is_view_v<Offset>>
 queryDispatch(Tag, Tree const &tree, ExecutionSpace const &space,
               Predicates const &predicates, Indices &indices, Offset &offset,
               Experimental::TraversalPolicy const &policy =
@@ -447,7 +424,7 @@ queryDispatch(Tag, Tree const &tree, ExecutionSpace const &space,
 template <typename Tag, typename Tree, typename ExecutionSpace,
           typename Predicates, typename OutputView, typename OffsetView,
           typename Callback>
-inline std::enable_if_t<is_tagged_post_callback<Callback>{}>
+inline std::enable_if_t<is_tagged_post_callback<Callback>::value>
 queryDispatch(Tag, Tree const &tree, ExecutionSpace const &space,
               Predicates const &predicates, Callback const &callback,
               OutputView &out, OffsetView &offset,
@@ -455,25 +432,29 @@ queryDispatch(Tag, Tree const &tree, ExecutionSpace const &space,
                   Experimental::TraversalPolicy())
 {
   using MemorySpace = typename Tree::memory_space;
-  Kokkos::View<int *, MemorySpace> indices(
+
+  Kokkos::View<typename Tree::value_type *, MemorySpace> indices(
       "ArborX::CrsGraphWrapper::query::indices", 0);
-  queryDispatch(Tag{}, tree, space, predicates, indices, offset, policy);
+  queryDispatch(Tag{}, tree, space, predicates, DefaultCallback{}, indices,
+                offset, policy);
   callback(predicates, offset, indices, out);
 }
 
-template <typename Callback, typename Predicates, typename OutputView>
-std::enable_if_t<!Kokkos::is_view<Callback>{} &&
-                 !is_tagged_post_callback<Callback>{}>
+template <typename Value, typename Callback, typename Predicates,
+          typename OutputView>
+std::enable_if_t<!Kokkos::is_view_v<Callback> &&
+                 !is_tagged_post_callback<Callback>::value>
 check_valid_callback_if_first_argument_is_not_a_view(
     Callback const &callback, Predicates const &predicates,
     OutputView const &out)
 {
-  check_valid_callback(callback, predicates, out);
+  check_valid_callback<Value>(callback, predicates, out);
 }
 
-template <typename Callback, typename Predicates, typename OutputView>
-std::enable_if_t<!Kokkos::is_view<Callback>{} &&
-                 is_tagged_post_callback<Callback>{}>
+template <typename Value, typename Callback, typename Predicates,
+          typename OutputView>
+std::enable_if_t<!Kokkos::is_view_v<Callback> &&
+                 is_tagged_post_callback<Callback>::value>
 check_valid_callback_if_first_argument_is_not_a_view(Callback const &,
                                                      Predicates const &,
                                                      OutputView const &)
@@ -481,8 +462,9 @@ check_valid_callback_if_first_argument_is_not_a_view(Callback const &,
   // TODO
 }
 
-template <typename View, typename Predicates, typename OutputView>
-std::enable_if_t<Kokkos::is_view<View>{}>
+template <typename Value, typename View, typename Predicates,
+          typename OutputView>
+std::enable_if_t<Kokkos::is_view_v<View>>
 check_valid_callback_if_first_argument_is_not_a_view(View const &,
                                                      Predicates const &,
                                                      OutputView const &)
