@@ -15,18 +15,20 @@
 #include <ArborX_DetailsAlgorithms.hpp> // equal
 #include <ArborX_DetailsKokkosExtArithmeticTraits.hpp>
 #include <ArborX_DetailsKokkosExtSwap.hpp>
+#include <ArborX_HyperTriangle.hpp>
 #include <ArborX_Point.hpp>
 #include <ArborX_Sphere.hpp>
-#include <ArborX_Triangle.hpp>
 
 #include <Kokkos_Macros.hpp>
+#if KOKKOS_VERSION >= 40200
+#include <Kokkos_Assert.hpp> // KOKKOS_ASSERT
+#else
+#include <impl/Kokkos_Error.hpp> // KOKKOS_ASSERT
+#endif
 
-#include <cassert>
 #include <cmath>
 
-namespace ArborX
-{
-namespace Experimental
+namespace ArborX::Experimental
 {
 
 struct Vector : private Point
@@ -40,8 +42,14 @@ struct Vector : private Point
   }
 };
 
-KOKKOS_INLINE_FUNCTION constexpr Vector makeVector(Point const &begin,
-                                                   Point const &end)
+template <typename Point1, typename Point2>
+KOKKOS_INLINE_FUNCTION constexpr std::enable_if_t<
+    GeometryTraits::is_point<Point1>::value &&
+        GeometryTraits::is_point<Point2>::value &&
+        GeometryTraits::dimension_v<Point1> == 3 &&
+        GeometryTraits::dimension_v<Point2> == 3,
+    Vector>
+makeVector(Point1 const &begin, Point2 const &end)
 {
   Vector v;
   for (int d = 0; d < 3; ++d)
@@ -119,7 +127,7 @@ private:
   KOKKOS_FUNCTION static void normalize(Vector &v)
   {
     auto const magv = norm(v);
-    assert(magv > 0);
+    KOKKOS_ASSERT(magv > 0);
     for (int d = 0; d < 3; ++d)
       v[d] /= magv;
   }
@@ -164,7 +172,8 @@ bool intersection(Ray const &ray, Box const &box, float &tmin, float &tmax)
   auto const &orig = ray.origin();
   auto const &dir = ray.direction();
 
-  constexpr auto inf = KokkosExt::ArithmeticTraits::infinity<float>::value;
+  constexpr auto inf =
+      Details::KokkosExt::ArithmeticTraits::infinity<float>::value;
   tmin = -inf;
   tmax = inf;
 
@@ -292,9 +301,12 @@ KOKKOS_INLINE_FUNCTION bool rayEdgeIntersect(Point const &edge_vertex_1,
 // when the ray and the triangle is coplanar.
 // In the paper, they just need the boolean return.
 KOKKOS_INLINE_FUNCTION
-bool intersection(Ray const &ray, Triangle const &triangle, float &tmin,
-                  float &tmax)
+bool intersection(Ray const &ray,
+                  ExperimentalHyperGeometry::Triangle<3> const &triangle,
+                  float &tmin, float &tmax)
 {
+  namespace KokkosExt = Details::KokkosExt;
+
   auto dir = ray.direction();
   // normalize the direction vector by its largest component.
   auto kz = findLargestComp(dir);
@@ -437,8 +449,9 @@ bool intersection(Ray const &ray, Triangle const &triangle, float &tmin,
   return false;
 } // namespace Experimental
 
-KOKKOS_INLINE_FUNCTION
-bool intersects(Ray const &ray, Triangle const &triangle)
+KOKKOS_INLINE_FUNCTION bool
+intersects(Ray const &ray,
+           ExperimentalHyperGeometry::Triangle<3> const &triangle)
 {
   float tmin;
   float tmax;
@@ -457,8 +470,9 @@ float distance(Ray const &ray, Box const &box)
   float tmin;
   float tmax;
   bool intersects = intersection(ray, box, tmin, tmax) && (tmax >= 0.f);
-  return intersects ? (tmin > 0.f ? tmin : 0.f)
-                    : KokkosExt::ArithmeticTraits::infinity<float>::value;
+  return intersects
+             ? (tmin > 0.f ? tmin : 0.f)
+             : Details::KokkosExt::ArithmeticTraits::infinity<float>::value;
 }
 
 // Solves a*x^2 + b*x + c = 0.
@@ -467,7 +481,7 @@ float distance(Ray const &ray, Box const &box)
 KOKKOS_INLINE_FUNCTION bool solveQuadratic(float const a, float const b,
                                            float const c, float &x1, float &x2)
 {
-  assert(a != 0);
+  KOKKOS_ASSERT(a != 0);
 
   auto const discriminant = b * b - 4 * a * c;
   if (discriminant < 0)
@@ -511,6 +525,8 @@ KOKKOS_INLINE_FUNCTION bool solveQuadratic(float const a, float const b,
 KOKKOS_INLINE_FUNCTION bool intersection(Ray const &ray, Sphere const &sphere,
                                          float &tmin, float &tmax)
 {
+  namespace KokkosExt = ArborX::Details::KokkosExt;
+
   auto const &r = sphere.radius();
 
   // Vector oc = (origin_of_ray - center_of_sphere)
@@ -539,6 +555,8 @@ KOKKOS_INLINE_FUNCTION void
 overlapDistance(Ray const &ray, Geometry const &geometry, float &length,
                 float &distance_to_origin)
 {
+  namespace KokkosExt = ArborX::Details::KokkosExt;
+
   float tmin;
   float tmax;
   if (intersection(ray, geometry, tmin, tmax) && (tmin <= tmax && tmax >= 0))
@@ -566,6 +584,22 @@ KOKKOS_INLINE_FUNCTION float overlapDistance(Ray const &ray,
   return length;
 }
 
-} // namespace Experimental
-} // namespace ArborX
+} // namespace ArborX::Experimental
+
+template <>
+struct ArborX::GeometryTraits::dimension<ArborX::Experimental::Ray>
+{
+  static constexpr int value = 3;
+};
+template <>
+struct ArborX::GeometryTraits::tag<ArborX::Experimental::Ray>
+{
+  using type = RayTag;
+};
+template <>
+struct ArborX::GeometryTraits::coordinate_type<ArborX::Experimental::Ray>
+{
+  using type = float;
+};
+
 #endif
