@@ -12,8 +12,7 @@
 #include "ArborXTest_Cloud.hpp"
 #include "ArborXTest_StdVectorToKokkosView.hpp"
 #include "ArborX_BoostRTreeHelpers.hpp"
-#include "ArborX_EnableDeviceTypes.hpp"           // ARBORX_DEVICE_TYPES
-#include <ArborX_DetailsKokkosExtClassLambda.hpp> // ARBORX_CLASS_LAMBDA
+#include "ArborX_EnableDeviceTypes.hpp" // ARBORX_DEVICE_TYPES
 #include <ArborX_DistributedTree.hpp>
 
 #include <boost/test/unit_test.hpp>
@@ -31,7 +30,6 @@
 namespace tt = boost::test_tools;
 
 using ArborX::PairIndexRank;
-using ArborX::Details::PairIndexRankAndDistance;
 
 BOOST_AUTO_TEST_CASE_TEMPLATE(hello_world, DeviceType, ARBORX_DEVICE_TYPES)
 {
@@ -134,6 +132,7 @@ BOOST_AUTO_TEST_CASE_TEMPLATE(hello_world, DeviceType, ARBORX_DEVICE_TYPES)
 
 BOOST_AUTO_TEST_CASE_TEMPLATE(empty_tree, DeviceType, ARBORX_DEVICE_TYPES)
 {
+  using Tree = ArborX::DistributedTree<typename DeviceType::memory_space>;
   using ExecutionSpace = typename DeviceType::execution_space;
 
   MPI_Comm comm = MPI_COMM_WORLD;
@@ -142,84 +141,81 @@ BOOST_AUTO_TEST_CASE_TEMPLATE(empty_tree, DeviceType, ARBORX_DEVICE_TYPES)
   int comm_size;
   MPI_Comm_size(comm, &comm_size);
 
-  auto const tree = makeDistributedTree<DeviceType>(comm, {});
-
-  BOOST_TEST(tree.empty());
-  BOOST_TEST(tree.size() == 0);
-
-  BOOST_TEST(ArborX::Details::equals(tree.bounds(), {}));
-
-  ARBORX_TEST_QUERY_TREE(ExecutionSpace{}, tree,
-                         makeIntersectsBoxQueries<DeviceType>({}),
-                         make_reference_solution<PairIndexRank>({}, {0}));
-
-  ARBORX_TEST_QUERY_TREE(ExecutionSpace{}, tree,
-                         makeIntersectsSphereQueries<DeviceType>({}),
-                         make_reference_solution<PairIndexRank>({}, {0}));
-
-  ARBORX_TEST_QUERY_TREE(ExecutionSpace{}, tree,
-                         makeNearestQueries<DeviceType>({}),
-                         make_reference_solution<PairIndexRank>({}, {0}));
-
-  ARBORX_TEST_QUERY_TREE_WITH_DISTANCE(
-      ExecutionSpace{}, tree, makeNearestQueries<DeviceType>({}),
-      make_reference_solution<PairIndexRankAndDistance>({}, {0}));
-
-  // Only rank 0 has a couple spatial queries with a spatial predicate
-  if (comm_rank == 0)
-  {
-    ARBORX_TEST_QUERY_TREE(
-        ExecutionSpace{}, tree, makeIntersectsBoxQueries<DeviceType>({{}, {}}),
-        make_reference_solution<PairIndexRank>({}, {0, 0, 0}));
-  }
-  else
+  Tree default_initialized;
+  Tree value_initialized{};
+  for (auto const &tree : {
+           default_initialized, value_initialized,
+           makeDistributedTree<DeviceType>(
+               comm, {}) // constructed with empty view of boxes
+       })
   {
+
+    BOOST_TEST(tree.empty());
+    BOOST_TEST(tree.size() == 0);
+
+    BOOST_TEST(ArborX::Details::equals(tree.bounds(), {}));
+
     ARBORX_TEST_QUERY_TREE(ExecutionSpace{}, tree,
                            makeIntersectsBoxQueries<DeviceType>({}),
                            make_reference_solution<PairIndexRank>({}, {0}));
-  }
 
-  // All ranks but rank 0 have a single query with a spatial predicate
-  if (comm_rank == 0)
-  {
     ARBORX_TEST_QUERY_TREE(ExecutionSpace{}, tree,
                            makeIntersectsSphereQueries<DeviceType>({}),
                            make_reference_solution<PairIndexRank>({}, {0}));
-  }
-  else
-  {
-    ARBORX_TEST_QUERY_TREE(
-        ExecutionSpace{}, tree,
-        makeIntersectsSphereQueries<DeviceType>({
-            {{{(float)comm_rank, 0.f, 0.f}}, (float)comm_size},
-        }),
-        make_reference_solution<PairIndexRank>({}, {0, 0}));
-  }
 
-  // All ranks but rank 0 have a single query with a nearest predicate
-  if (comm_rank == 0)
-  {
     ARBORX_TEST_QUERY_TREE(ExecutionSpace{}, tree,
                            makeNearestQueries<DeviceType>({}),
                            make_reference_solution<PairIndexRank>({}, {0}));
-  }
-  else
-  {
-    ARBORX_TEST_QUERY_TREE(ExecutionSpace{}, tree,
-                           makeNearestQueries<DeviceType>({
-                               {{{0., 0., 0.}}, comm_rank},
-                           }),
-                           make_reference_solution<PairIndexRank>({}, {0, 0}));
-  }
 
-  // All ranks have a single query with a nearest predicate (this version
-  // returns distances as well)
-  ARBORX_TEST_QUERY_TREE_WITH_DISTANCE(
-      ExecutionSpace{}, tree,
-      makeNearestQueries<DeviceType>({
-          {{{0., 0., 0.}}, comm_size},
-      }),
-      make_reference_solution<PairIndexRankAndDistance>({}, {0, 0}));
+    // Only rank 0 has a couple spatial queries with a spatial predicate
+    if (comm_rank == 0)
+    {
+      ARBORX_TEST_QUERY_TREE(
+          ExecutionSpace{}, tree,
+          makeIntersectsBoxQueries<DeviceType>({{}, {}}),
+          make_reference_solution<PairIndexRank>({}, {0, 0, 0}));
+    }
+    else
+    {
+      ARBORX_TEST_QUERY_TREE(ExecutionSpace{}, tree,
+                             makeIntersectsBoxQueries<DeviceType>({}),
+                             make_reference_solution<PairIndexRank>({}, {0}));
+    }
+
+    // All ranks but rank 0 have a single query with a spatial predicate
+    if (comm_rank == 0)
+    {
+      ARBORX_TEST_QUERY_TREE(ExecutionSpace{}, tree,
+                             makeIntersectsSphereQueries<DeviceType>({}),
+                             make_reference_solution<PairIndexRank>({}, {0}));
+    }
+    else
+    {
+      ARBORX_TEST_QUERY_TREE(
+          ExecutionSpace{}, tree,
+          makeIntersectsSphereQueries<DeviceType>({
+              {{{(float)comm_rank, 0.f, 0.f}}, (float)comm_size},
+          }),
+          make_reference_solution<PairIndexRank>({}, {0, 0}));
+    }
+
+    // All ranks but rank 0 have a single query with a nearest predicate
+    if (comm_rank == 0)
+    {
+      ARBORX_TEST_QUERY_TREE(ExecutionSpace{}, tree,
+                             makeNearestQueries<DeviceType>({}),
+                             make_reference_solution<PairIndexRank>({}, {0}));
+    }
+    else
+    {
+      ARBORX_TEST_QUERY_TREE(
+          ExecutionSpace{}, tree,
+          makeNearestQueries<DeviceType>({
+              {{{0., 0., 0.}}, comm_rank},
+          }),
+          make_reference_solution<PairIndexRank>({}, {0, 0}));
+    }
+  }
 }
 
 BOOST_AUTO_TEST_CASE_TEMPLATE(unique_leaf_on_rank_0, DeviceType,
@@ -260,10 +256,6 @@ BOOST_AUTO_TEST_CASE_TEMPLATE(unique_leaf_on_rank_0, DeviceType,
                          makeNearestQueries<DeviceType>({}),
                          make_reference_solution<PairIndexRank>({}, {0}));
 
-  ARBORX_TEST_QUERY_TREE_WITH_DISTANCE(
-      ExecutionSpace{}, tree, makeNearestQueries<DeviceType>({}),
-      make_reference_solution<PairIndexRankAndDistance>({}, {0}));
-
   // Querying for more neighbors than there are leaves in the tree
   ARBORX_TEST_QUERY_TREE(
       ExecutionSpace{}, tree,
@@ -592,7 +584,7 @@ struct CustomPostCallbackWithAttachment
     auto const n = offset.extent(0) - 1;
     Kokkos::realloc(out, in.extent(0));
     Kokkos::parallel_for(
-        Kokkos::RangePolicy<ExecutionSpace>(0, n), ARBORX_CLASS_LAMBDA(int i) {
+        Kokkos::RangePolicy<ExecutionSpace>(0, n), KOKKOS_CLASS_LAMBDA(int i) {
           auto data = ArborX::getData(queries(i));
           for (int j = offset(i); j < offset(i + 1); ++j)
           {
