@@ -9,8 +9,8 @@
  * SPDX-License-Identifier: BSD-3-Clause                                    *
  ****************************************************************************/
 #include <ArborX_Box.hpp>
+#include <ArborX_HyperTriangle.hpp>
 #include <ArborX_Ray.hpp>
-#include <ArborX_Triangle.hpp>
 
 #include <boost/test/unit_test.hpp>
 
@@ -172,7 +172,6 @@ BOOST_AUTO_TEST_CASE(intersects_box)
   {                                                                            \
     float t0;                                                                  \
     float t1;                                                                  \
-    constexpr auto inf = KokkosExt::ArithmeticTraits::infinity<float>::value;  \
     BOOST_TEST(!ArborX::Experimental::intersection(ray, box, t0, t1));         \
     BOOST_TEST((t0 == inf || t1 == -inf));                                     \
   } while (false)
@@ -182,6 +181,13 @@ BOOST_AUTO_TEST_CASE(ray_box_intersection, *boost::unit_test::tolerance(1e-6f))
   using ArborX::Box;
   using ArborX::Experimental::Ray;
 
+  namespace KokkosExt = ArborX::Details::KokkosExt;
+#ifdef _MSC_VER
+  auto const inf = KokkosExt::ArithmeticTraits::infinity<float>::value;
+#else
+  constexpr auto inf = KokkosExt::ArithmeticTraits::infinity<float>::value;
+#endif
+
   constexpr Box unit_box{{0, 0, 0}, {1, 1, 1}};
 
   auto const sqrtf_5 = std::sqrt(5.f);
@@ -250,9 +256,16 @@ BOOST_AUTO_TEST_CASE(ray_box_distance)
 {
   using ArborX::Box;
   using ArborX::Experimental::Ray;
-
+  // use const instead of constexpr because MSVC shows error(error:expression
+  // must have a constant value)
   constexpr Box unit_box{{0, 0, 0}, {1, 1, 1}};
+
+  namespace KokkosExt = ArborX::Details::KokkosExt;
+#ifdef _MSC_VER
+  auto const inf = KokkosExt::ArithmeticTraits::infinity<float>::value;
+#else
   constexpr auto inf = KokkosExt::ArithmeticTraits::infinity<float>::value;
+#endif
 
   // clang-format off
   // origin is within the box
@@ -347,7 +360,6 @@ BOOST_AUTO_TEST_CASE(overlap_distance_sphere,
   {                                                                            \
     float t0;                                                                  \
     float t1;                                                                  \
-    constexpr auto inf = KokkosExt::ArithmeticTraits::infinity<float>::value;  \
     BOOST_TEST(!ArborX::Experimental::intersection(ray, sphere, t0, t1));      \
     BOOST_TEST((t0 == inf && t1 == -inf));                                     \
   } while (false)
@@ -359,10 +371,16 @@ BOOST_AUTO_TEST_CASE(ray_sphere_intersection,
   using ArborX::Experimental::Ray;
 
   constexpr Sphere unit_sphere{{0, 0, 0}, 1};
-
   auto const sqrtf_3 = std::sqrt(3.f);
   auto const sqrtf_2 = std::sqrt(2.f);
 
+  namespace KokkosExt = ArborX::Details::KokkosExt;
+#ifdef _MSC_VER
+  auto const inf = KokkosExt::ArithmeticTraits::infinity<float>::value;
+#else
+  constexpr auto inf = KokkosExt::ArithmeticTraits::infinity<float>::value;
+#endif
+
   // clang-format off
   // hit center of the sphere
   ARBORX_TEST_RAY_SPHERE_INTERSECTION((Ray{{-2, 0, 0}, {1, 0, 0}}), unit_sphere, 1.f, 3.f);
@@ -396,8 +414,10 @@ BOOST_AUTO_TEST_CASE(ray_sphere_intersection,
 BOOST_AUTO_TEST_CASE(intersects_triangle)
 {
   using ArborX::Experimental::Ray;
-  using ArborX::Experimental::Triangle;
-  constexpr Triangle unit_triangle{{0, 0, 0}, {1, 0, 0}, {0, 1, 0}};
+  using ArborX::ExperimentalHyperGeometry::Point;
+  using ArborX::ExperimentalHyperGeometry::Triangle;
+  constexpr Triangle unit_triangle{Point{0, 0, 0}, Point{1, 0, 0},
+                                   Point{0, 1, 0}};
 
   BOOST_TEST(intersects(Ray{{.1, .2, .3}, {0, 0, -1}}, unit_triangle));
   BOOST_TEST(intersects(Ray{{1.1, 1.2, 1}, {-1, -1, -1}}, unit_triangle));
@@ -443,7 +463,8 @@ BOOST_AUTO_TEST_CASE(intersects_triangle)
   // ray in a plane parallel to the triangle
   BOOST_TEST(!intersects(Ray{{-0.1, 0, 1}, {1, 0, 0}}, unit_triangle));
 
-  constexpr Triangle tilted_triangle{{0, 0, 0}, {2, 0, 1}, {0, 2, 1}};
+  constexpr Triangle tilted_triangle{Point{0, 0, 0}, Point{2, 0, 1},
+                                     Point{0, 2, 1}};
 
   // ray in the same plane as the triangle
   BOOST_TEST(!intersects(Ray{{10, 0, 0}, {1, 1, 1}}, tilted_triangle));
@@ -468,7 +489,6 @@ BOOST_AUTO_TEST_CASE(intersects_triangle)
   {                                                                            \
     float t0;                                                                  \
     float t1;                                                                  \
-    constexpr auto inf = KokkosExt::ArithmeticTraits::infinity<float>::value;  \
     BOOST_TEST(!ArborX::Experimental::intersection(ray, triangle, t0, t1));    \
     BOOST_TEST((t0 == inf && t1 == -inf));                                     \
   } while (false)
@@ -476,12 +496,21 @@ BOOST_AUTO_TEST_CASE(intersects_triangle)
 BOOST_AUTO_TEST_CASE(ray_triangle_intersection,
                      *boost::unit_test::tolerance(2e-6f))
 {
-  using ArborX::Point;
   using ArborX::Experimental::Ray;
-  using ArborX::Experimental::Triangle;
+  using ArborX::ExperimentalHyperGeometry::Point;
+  using ArborX::ExperimentalHyperGeometry::Triangle;
 
-  constexpr Triangle unit_triangle{{0, 0, 0}, {1, 0, 0}, {0, 1, 0}};
-  constexpr Triangle narrow_triangle{{0.5, 0.5, 0}, {0.24, 0.74, 0}, {0, 1, 0}};
+  namespace KokkosExt = ArborX::Details::KokkosExt;
+#ifdef _MSC_VER
+  auto const inf = KokkosExt::ArithmeticTraits::infinity<float>::value;
+#else
+  constexpr auto inf = KokkosExt::ArithmeticTraits::infinity<float>::value;
+#endif
+
+  constexpr Triangle unit_triangle{Point{0, 0, 0}, Point{1, 0, 0},
+                                   Point{0, 1, 0}};
+  constexpr Triangle narrow_triangle{Point{0.5f, 0.5f, 0},
+                                     Point{0.24f, 0.74f, 0}, Point{0, 1.f, 0}};
 
   auto const sqrtf_3 = std::sqrt(3.f);
   auto const sqrtf_2 = std::sqrt(2.f);
@@ -534,11 +563,11 @@ BOOST_AUTO_TEST_CASE(ray_triangle_intersection,
   // guarantees watertightness along the edges and at the vertices". I assume the below
   // test would fail the "Plucker coordinates"-based algorithm, thus it is necessary to
   // keep them here to show the watertightness of the current algorithm implemented.
-  constexpr Point O{1.0, 1.0, 1.0};
-  constexpr Point A{2.0, 2.0, 0.0};
-  constexpr Point B{2.0, -1.0, 0.0};
-  constexpr Point C{-1.0, -1.0, 0.0};
-  constexpr Point D{-1.0, 2.0f, 0.0f};
+  constexpr Point O{1.f, 1.f, 1.f};
+  constexpr Point A{2.f, 2.f, 0};
+  constexpr Point B{2.f, -1.f, 0};
+  constexpr Point C{-1.f, -1.f, 0};
+  constexpr Point D{-1.f, 2.f, 0};
 
   constexpr Triangle triangle_up{D, A, O};
   constexpr Triangle triangle_right{O, A, B};
@@ -571,12 +600,12 @@ BOOST_AUTO_TEST_CASE(ray_triangle_intersection,
   // between the origin of the ray and the vertices.
   // These tests will fail if there is no normalization.
   float const size_s = 0.0001;
-  Triangle small_triangle{{-size_s, 0, 0}, {0, size_s, 0}, {0, 0, size_s}};
+  Triangle small_triangle{Point{-size_s, 0, 0}, Point{0, size_s, 0}, Point{0, 0, size_s}};
   ARBORX_TEST_RAY_TRIANGLE_INTERSECTION((Ray{{0.0, 0.0, 0.0}, {-size_s, size_s, size_s}}), small_triangle, size_s/sqrtf_3, size_s/sqrtf_3);
   ARBORX_TEST_RAY_TRIANGLE_INTERSECTION((Ray{{-2.f*size_s, -size_s, 0.0}, {size_s, size_s, 0}}), small_triangle, sqrtf_2*size_s, 2.f*sqrtf_2*size_s);
 
   float const size_l = 10000;
-  Triangle large_triangle{{-size_l, 0, 0}, {0, 10000, 0}, {0, 0, 10000}};
+  Triangle large_triangle{Point{-size_l, 0, 0}, Point{0, 10000, 0}, Point{0, 0, 10000}};
   ARBORX_TEST_RAY_TRIANGLE_INTERSECTION((Ray{{0.0, 0.0, 0.0}, {-size_l, size_l, size_l}}), large_triangle, size_l/sqrtf_3, size_l/sqrtf_3);
   ARBORX_TEST_RAY_TRIANGLE_INTERSECTION((Ray{{-2.f*size_l, -size_l, 0.0}, {size_l, size_l, 0}}), large_triangle, sqrtf_2*size_l, 2.f*sqrtf_2*size_l);
 
@@ -590,8 +619,9 @@ BOOST_AUTO_TEST_CASE(make_euclidean_vector)
 {
   using ArborX::Experimental::makeVector;
   using ArborX::Experimental::Vector;
-  static_assert(makeVector({0, 0, 0}, {1, 2, 3}) == Vector{1, 2, 3});
-  static_assert(makeVector({1, 2, 3}, {4, 5, 6}) == Vector{3, 3, 3});
+  using ArborX::ExperimentalHyperGeometry::Point;
+  static_assert(makeVector(Point{0, 0, 0}, Point{1, 2, 3}) == Vector{1, 2, 3});
+  static_assert(makeVector(Point{1, 2, 3}, Point{4, 5, 6}) == Vector{3, 3, 3});
 }
 
 BOOST_AUTO_TEST_CASE(dot_product)
